@@ -269,7 +269,7 @@ pub mod str_option {
         match *value {
             Some(ref decimal) => {
                 let decimal = crate::str::to_str_internal(decimal, true, None);
-                serializer.serialize_some(decimal.0.as_ref())
+                serializer.serialize_some::<str>(decimal.0.as_ref())
             }
             None => serializer.serialize_none(),
         }
@@ -339,6 +339,7 @@ impl<'de> serde::de::Visitor<'de> for DecimalVisitor {
         }
     }
 
+    #[cfg(not(feature = "serde-with-arbitrary-precision"))]
     fn visit_f64<E>(self, value: f64) -> Result<Decimal, E>
     where
         E: serde::de::Error,
@@ -346,6 +347,15 @@ impl<'de> serde::de::Visitor<'de> for DecimalVisitor {
         Decimal::from_str(&value.to_string()).map_err(|_| E::invalid_value(Unexpected::Float(value), &self))
     }
 
+    #[cfg(feature = "serde-with-arbitrary-precision")]
+    fn visit_f64<E>(self, value: f64) -> Result<Decimal, E>
+    where
+        E: serde::de::Error,
+    {
+        Decimal::from_str(ryu::Buffer::new().format_finite(value))
+            .map_err(|_| E::invalid_value(Unexpected::Float(value), &self))
+    }
+
     fn visit_str<E>(self, value: &str) -> Result<Decimal, E>
     where
         E: serde::de::Error,
@@ -370,8 +380,10 @@ impl<'de> serde::de::Visitor<'de> for DecimalVisitor {
     }
 }
 
+#[cfg(any(feature = "serde-with-float", feature = "serde-with-arbitrary-precision"))]
 struct OptionDecimalVisitor;
 
+#[cfg(any(feature = "serde-with-float", feature = "serde-with-arbitrary-precision"))]
 impl<'de> serde::de::Visitor<'de> for OptionDecimalVisitor {
     type Value = Option<Decimal>;
 
@@ -644,6 +656,33 @@ mod test {
         assert_eq!(record.amount, deserialized.amount);
     }
 
+    #[test]
+    #[cfg(all(feature = "serde-float", feature = "serde-arbitrary-precision"))]
+    fn serialize_whole_number_decimal() {
+        let data = [
+            ("0", "0"),
+            ("1.0", "1.0"),
+            ("0.00", "0.00"),
+            ("1.234", "1.234"),
+            ("3.14159", "3.14159"),
+            ("-3.14159", "-3.14159"),
+            ("1234567890123.4567890", "1234567890123.4567890"),
+            ("-1234567890123.4567890", "-1234567890123.4567890"),
+        ];
+
+        for &(value, expected) in data.iter() {
+            let record = Record {
+                amount: Decimal::from_str(value).unwrap(),
+            };
+
+            let serialized = serde_json::to_string(&record).unwrap();
+            let value: serde_json::Value = serde_json::from_str(&serialized).unwrap();
+            let deserialized: Record = serde_json::from_value(value).unwrap();
+
+            assert_eq!(expected, deserialized.amount.to_string());
+        }
+    }
+
     #[test]
     #[cfg(all(feature = "serde-str", not(feature = "serde-float")))]
     fn serialize_decimal_roundtrip() {
